@@ -30,17 +30,19 @@
 #include <fstream>
 #include <vector>
 #include <string>
+#include <string.h>
 #include <map>
 #include <algorithm>
 
+#include <soul/soul_patch.h>
+#include <soul/patch/helper_classes/soul_patch_Utilities.h>
+
 #include "faust/dsp/dsp.h"
 #include "faust/midi/midi.h"
-#include "faust/GUI/UI.h"
-#include "faust/GUI/JSONUIDecoder.h"
+#include "faust/gui/UI.h"
+#include "faust/gui/JSONUIDecoder.h"
 #include "faust/dsp/libfaust.h"
 
-#include "soul/API/soul_patch.h"
-
 class soul_dsp_factory;
 
 /**
@@ -51,6 +53,17 @@ class soulpatch_dsp : public dsp {
     
     private:
     
+        /*
+        struct ConsolePrinter : public soul::patch::RefCountHelper<soul::patch::ConsoleMessageHandler, ConsolePrinter>
+        {
+            // Called when a message is sent to the console by the program.
+            void handleConsoleMessage(uint64_t sampleCount, const char* endpointName, const char* message)
+            {
+                std::cout << sampleCount << " " << endpointName << ": " << message << std::endl;
+            }
+        };
+        */
+
         bool startWith(const std::string& str, const std::string& prefix)
         {
             return (str.substr(0, prefix.size()) == prefix);
@@ -96,7 +109,7 @@ class soulpatch_dsp : public dsp {
         std::vector<ZoneParam*> fInputsControl;
         std::vector<ZoneParam*> fOutputsControl;
     
-        soul::patch::PatchPlayer::Ptr fPlayer;
+        soul::patch::PatchPlayer* fPlayer;
         soul::patch::PatchPlayerConfiguration fConfig;
         soul::patch::PatchPlayer::RenderContext fRenderContext;
     
@@ -109,8 +122,8 @@ class soulpatch_dsp : public dsp {
     
         // MIDI handling
         midi_handler* fMIDIHander;
-        std::vector<soul::patch::MIDIMessage> fMIDIInputMessages;
-        std::vector<soul::patch::MIDIMessage> fMIDIOutputMessages;
+        std::vector<soul::MIDIEvent> fMIDIInputMessages;
+        std::vector<soul::MIDIEvent> fMIDIOutputMessages;
     
     public:
 
@@ -182,7 +195,7 @@ class soulpatch_dsp : public dsp {
                         for (int j = 0; j < names.size(); j++) {
                             std::string key = names[j];
                             if (key == "unit") {
-                                ui_interface->declare(zone, key.c_str(), params[i]->getProperty(names[j]));
+                                ui_interface->declare(zone, key.c_str(), params[i]->getProperty(names[j])->getCharPointer());
                             }
                         }
                         
@@ -194,6 +207,7 @@ class soulpatch_dsp : public dsp {
                                                               params[i]->maxValue, params[i]->step);
                         }
                     }
+                    
                     ui_interface->closeBox();
                 }
                 {
@@ -212,10 +226,10 @@ class soulpatch_dsp : public dsp {
                         for (int j = 0; j < names.size(); j++) {
                             std::string key = names[j];
                             if (key == "unit") {
-                                ui_interface->declare(zone, key.c_str(), params[i]->getProperty(names[j]));
+                                ui_interface->declare(zone, key.c_str(), params[i]->getProperty(names[j])->getCharPointer());
                             }
                         }
-                        if !(checkParam(params[i]->getPropertyNames(), "hidden")) {
+                        if (!(checkParam(params[i]->getPropertyNames(), "hidden"))) {
                             ui_interface->addVerticalBargraph(params[i]->name->getCharPointer(), zone, params[i]->minValue, params[i]->maxValue);
                         }
                     }
@@ -289,10 +303,14 @@ class soulpatch_dsp : public dsp {
                 fRenderContext.outgoingMIDI = std::addressof(fMIDIOutputMessages[0]);
                 fRenderContext.maximumMIDIMessagesOut = (uint32_t)fMIDIOutputMessages.size();
                 fRenderContext.numMIDIMessagesOut = 0;
+            } else {
+                fRenderContext.incomingMIDI = nullptr;
+                fRenderContext.incomingMIDI = nullptr;
+                fRenderContext.numMIDIMessagesIn = 0;
             }
         
             // Setup audio buffers
-            fRenderContext.inputChannels = (const float**)inputs;
+            fRenderContext.inputChannels = (const FAUSTFLOAT**)inputs;
             fRenderContext.outputChannels = outputs;
             fRenderContext.numFrames = count;
             
@@ -336,13 +354,13 @@ class soul_dsp_factory : public dsp_factory {
     
         struct FaustSOULFile : public soul::patch::VirtualFile {
             
-            virtual soul::patch::String::Ptr getName() { return {}; }
+            virtual soul::patch::String* getName() { return {}; }
             
-            virtual soul::patch::String::Ptr getAbsolutePath() { return {}; }
+            virtual soul::patch::String* getAbsolutePath() { return {}; }
             
-            virtual soul::patch::VirtualFile::Ptr getParent() { return {}; }
+            virtual soul::patch::VirtualFile* getParent() { return {}; }
             
-            virtual soul::patch::VirtualFile::Ptr getChildFile (const char* subPath) { return {}; }
+            virtual soul::patch::VirtualFile* getChildFile (const char* subPath) { return {}; }
             
             virtual int64_t getSize() { return 0; }
             
@@ -354,7 +372,7 @@ class soul_dsp_factory : public dsp_factory {
     
         struct FaustSourceFilePreprocessor : public soul::patch::SourceFilePreprocessor {
             
-            virtual soul::patch::VirtualFile::Ptr preprocessSourceFile (soul::patch::VirtualFile& inputFile) override
+            virtual soul::patch::VirtualFile* preprocessSourceFile (soul::patch::VirtualFile& inputFile) override
             {
                 return {};
             }
@@ -363,7 +381,7 @@ class soul_dsp_factory : public dsp_factory {
     
         std::string fPath;
         soul::patch::PatchInstance::Ptr fPatch;
-        soul::patch::Description fDescription;
+        soul::patch::Description* fDescription;
         FaustSourceFilePreprocessor::Ptr fProcessor;
     
     public:
@@ -398,7 +416,7 @@ class soul_dsp_factory : public dsp_factory {
     
         virtual ~soul_dsp_factory() {}
         
-        virtual std::string getName() { return fDescription.name; }
+        virtual std::string getName() { return fDescription->name; }
         virtual std::string getSHAKey() { return ""; }
         virtual std::string getDSPCode() { return ""; }
         virtual std::string getCompileOptions() { return ""; }
@@ -421,6 +439,7 @@ soulpatch_dsp::soulpatch_dsp(soul_dsp_factory* factory, std::string& error_msg)
     fFactory = factory;
     fDecoder = nullptr;
     fMIDIHander = nullptr;
+    memset(&fRenderContext, 0, sizeof(fRenderContext));
     
     fConfig.sampleRate = 44100;
     fConfig.maxFramesPerBlock = 4096;
@@ -492,6 +511,7 @@ soul_dsp_factory* createSOULDSPFactoryFromFile(const std::string& filename,
 {
     try {
         soul_dsp_factory* factory = new soul_dsp_factory(filename, error_msg);
+        // Check that SOUL patch compilation works
         soulpatch_dsp dummy(factory, error_msg);
         return factory;
     } catch (...) {
@@ -568,20 +588,23 @@ class faust_soul_parser  {
             return faust_blocks;
         }
     
-        std::string generateSOULBlock(const std::string& name, const std::string& code)
+        std::string generateSOULBlock(const std::string& name, const std::string& code, int argc, const char* argv[])
         {
-            int argc = 0;
-            const char* argv[16];
-            argv[argc++] = "-lang";
-            argv[argc++] = "soul";
-            argv[argc++] = "-cn";
-            argv[argc++] = name.c_str();
-            argv[argc++] = "-o";
-            argv[argc++] = "/var/tmp/exp.soul";
-            argv[argc] = nullptr;  // NULL terminated argv
+            int argc1 = 0;
+            const char* argv1[64];
+            argv1[argc1++] = "-lang";
+            argv1[argc1++] = "soul";
+            argv1[argc1++] = "-cn";
+            argv1[argc1++] = name.c_str();
+            argv1[argc1++] = "-o";
+            argv1[argc1++] = "/var/tmp/exp.soul";
+            for (int i = 0; i < argc; i++) {
+                argv1[argc1++] = argv[i];
+            }
+            argv1[argc1] = nullptr;  // NULL terminated argv
             
             std::string error_msg;
-            bool res = generateAuxFilesFromString(name, code, argc, argv, error_msg);
+            bool res = generateAuxFilesFromString(name, code, argc1, argv1, error_msg);
             
             if (res) {
                 std::ifstream soul_file("/var/tmp/exp.soul");
@@ -601,7 +624,7 @@ class faust_soul_parser  {
         ~faust_soul_parser()
         {}
  
-        bool parseSOULFile(const std::string& input, const std::string& output)
+        bool parseSOULFile(const std::string& input, const std::string& output, int argc, const char* argv[])
         {
             std::ifstream reader(input.c_str());
             if (reader.is_open()) {
@@ -615,7 +638,7 @@ class faust_soul_parser  {
                 
                 // Write all Faust blocks translated to SOUL
                 for (auto& it : faust_blocks) {
-                    std::string block = generateSOULBlock(it.first, it.second);
+                    std::string block = generateSOULBlock(it.first, it.second, argc, argv);
                     if (block == "") return false;
                     output_file << block;
                 }
@@ -630,18 +653,21 @@ class faust_soul_parser  {
             }
         }
     
-        bool generateSOULFile(const std::string& input, const std::string& output)
+        bool generateSOULFile(const std::string& input, const std::string& output, int argc, const char* argv[])
         {
-            int argc = 0;
-            const char* argv[16];
-            argv[argc++] = "-lang";
-            argv[argc++] = "soul";
-            argv[argc++] = "-o";
-            argv[argc++] = output.c_str();
-            argv[argc] = nullptr;  // NULL terminated argv
+            int argc1 = 0;
+            const char* argv1[64];
+            argv1[argc1++] = "-lang";
+            argv1[argc1++] = "soul";
+            argv1[argc1++] = "-o";
+            argv1[argc1++] = output.c_str();
+            for (int i = 0; i < argc; i++) {
+                argv1[argc1++] = argv[i];
+            }
+            argv1[argc1] = nullptr;  // NULL terminated argv
             
             std::string error_msg;
-            bool res = generateAuxFilesFromFile(input, argc, argv, error_msg);
+            bool res = generateAuxFilesFromFile(input, argc1, argv1, error_msg);
             if (!res) {
                 std::cerr << "ERROR : generateAuxFilesFromFile " << error_msg;
             }
