@@ -249,21 +249,11 @@ class APIUI : public PathBuilder, public Meta, public UI
 
         virtual ~APIUI()
         {
-            std::vector<ValueConverter*>::iterator it1;
-            for (it1 = fConversion.begin(); it1 != fConversion.end(); it1++) {
-                delete(*it1);
-            }
-
-            std::vector<ZoneControl*>::iterator it2;
+            for (auto& it : fConversion) delete it;
             for (int i = 0; i < 3; i++) {
-                for (it2 = fAcc[i].begin(); it2 != fAcc[i].end(); it2++) {
-                    delete(*it2);
-                }
-                for (it2 = fGyr[i].begin(); it2 != fGyr[i].end(); it2++) {
-                    delete(*it2);
-                }
+                for (auto& it : fAcc[i]) delete it;
+                for (auto& it : fGyr[i]) delete it;
             }
-            
             delete fRedReader;
             delete fGreenReader;
             delete fBlueReader;
@@ -271,10 +261,10 @@ class APIUI : public PathBuilder, public Meta, public UI
     
         // -- widget's layouts
 
-        virtual void openTabBox(const char* label)          { pushLabel(label); }
-        virtual void openHorizontalBox(const char* label)   { pushLabel(label); }
-        virtual void openVerticalBox(const char* label)     { pushLabel(label); }
-        virtual void closeBox()                             { popLabel(); }
+        virtual void openTabBox(const char* label) { pushLabel(label); }
+        virtual void openHorizontalBox(const char* label) { pushLabel(label); }
+        virtual void openVerticalBox(const char* label) { pushLabel(label); }
+        virtual void closeBox() { popLabel(); }
 
         // -- active widgets
 
@@ -323,7 +313,7 @@ class APIUI : public PathBuilder, public Meta, public UI
 
         virtual void declare(FAUSTFLOAT* zone, const char* key, const char* val)
         {
-            // Keep matadata
+            // Keep metadata
             fCurrentMetadata[key] = val;
             
             if (strcmp(key, "scale") == 0) {
@@ -370,9 +360,8 @@ class APIUI : public PathBuilder, public Meta, public UI
         {
             std::map<const char*, const char*> res;
             std::map<std::string, std::string> metadata = fMetaData[p];
-            std::map<std::string, std::string>::iterator it;
-            for (it = metadata.begin(); it != metadata.end(); ++it) {
-                res[(*it).first.c_str()] = (*it).second.c_str();
+            for (auto it : metadata) {
+                res[it.first.c_str()] = it.second.c_str();
             }
             return res;
         }
@@ -387,8 +376,20 @@ class APIUI : public PathBuilder, public Meta, public UI
         FAUSTFLOAT getParamInit(int p) { return fInit[p]; }
 
         FAUSTFLOAT* getParamZone(int p) { return fZone[p]; }
+    
         FAUSTFLOAT getParamValue(int p) { return *fZone[p]; }
+        FAUSTFLOAT getParamValue(const char* path)
+        {
+            int index = getParamIndex(path);
+            return (index >= 0) ? getParamValue(index) : FAUSTFLOAT(0);
+        }
+    
         void setParamValue(int p, FAUSTFLOAT v) { *fZone[p] = v; }
+        void setParamValue(const char* path, FAUSTFLOAT v)
+        {
+            int index = getParamIndex(path);
+            if (index >= 0) setParamValue(index, v);
+        }
 
         double getParamRatio(int p) { return fConversion[p]->faust2ui(*fZone[p]); }
         void setParamRatio(int p, double r) { *fZone[p] = fConversion[p]->ui2faust(r); }
@@ -432,7 +433,7 @@ class APIUI : public PathBuilder, public Meta, public UI
         }
    
         /**
-         * Set a new value coming from an accelerometer, propagate it to all relevant float* zones.
+         * Set a new value coming from an accelerometer, propagate it to all relevant FAUSTFLOAT* zones.
          *
          * @param acc - 0 for X accelerometer, 1 for Y accelerometer, 2 for Z accelerometer
          * @param value - the new value
@@ -510,7 +511,7 @@ class APIUI : public PathBuilder, public Meta, public UI
         }
     
         /**
-         * Set a new value coming from an gyroscope, propagate it to all relevant float* zones.
+         * Set a new value coming from an gyroscope, propagate it to all relevant FAUSTFLOAT* zones.
          *
          * @param gyr - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope
          * @param value - the new value
@@ -522,6 +523,30 @@ class APIUI : public PathBuilder, public Meta, public UI
                 fGyr[gyr][i]->update(value);
             }
         }
+    
+        /**
+         * Get the number of FAUSTFLOAT* zones controlled with the accelerometer
+         *
+         * @param acc - 0 for X accelerometer, 1 for Y accelerometer, 2 for Z accelerometer
+         * @return the number of zones
+         *
+         */
+        int getAccCount(int acc)
+        {
+            return (acc >= 0 && acc < 3) ? int(fAcc[acc].size()) : 0;
+        }
+    
+        /**
+         * Get the number of FAUSTFLOAT* zones controlled with the gyroscope
+         *
+         * @param gyr - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope
+         * @param the number of zones
+         *
+         */
+        int getGyrCount(int gyr)
+        {
+            return (gyr >= 0 && gyr < 3) ? int(fGyr[gyr].size()) : 0;
+        }
    
         // getScreenColor() : -1 means no screen color control (no screencolor metadata found)
         // otherwise return 0x00RRGGBB a ready to use color
@@ -536,7 +561,7 @@ class APIUI : public PathBuilder, public Meta, public UI
                 return -1;
             }
         }
-
+ 
 };
 
 #endif
