@@ -153,7 +153,7 @@ void CCodeContainer::produceClass()
     *fOut << "extern \"C\" {" << endl;
     *fOut << "#endif" << endl;
     tab(n, *fOut);
-
+  
     // Libraries
     printLibrary(*fOut);
     printIncludeFile(*fOut);
@@ -351,22 +351,16 @@ void CCodeContainer::produceClass()
 
     // Compute
     generateCompute(n);
-    tab(n, *fOut);
-
+  
     // Generate user interface macros if needed
     printMacros(*fOut, n);
 
+    tab(n, *fOut);
     *fOut << "#ifdef __cplusplus" << endl;
     *fOut << "}" << endl;
     *fOut << "#endif" << endl;
 }
 
-void CScalarOneSampleCodeContainer::produceInternal()
-{
-    generateGlobalDeclarations(fCodeProducer);
-    generateDeclarations(fCodeProducer);
-}
-
 void CScalarOneSampleCodeContainer::produceClass()
 {
     int n = 0;
@@ -377,15 +371,19 @@ void CScalarOneSampleCodeContainer::produceClass()
     *fOut << "#endif" << endl;
     tab(n, *fOut);
     
+    *fOut << "#if defined(_WIN32)" << endl;
+    *fOut << "#define RESTRICT __restrict" << endl;
+    *fOut << "#else" << endl;
+    *fOut << "#define RESTRICT __restrict__" << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+
     // Libraries
     printLibrary(*fOut);
     printIncludeFile(*fOut);
     
     // Sub containers
     mergeSubContainers();
-  
-    // Sub containers
-    generateSubContainers();
  
     // Functions
     tab(n, *fOut);
@@ -432,6 +430,10 @@ void CScalarOneSampleCodeContainer::produceClass()
     *fOut << "#define FAUST_INT_ZONE " << static_cast<CInstVisitor1*>(fCodeProducer)->getIntZoneSize() << endl;
     *fOut << "#define FAUST_FLOAT_ZONE " << static_cast<CInstVisitor1*>(fCodeProducer)->getRealZoneSize();
     
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "#ifndef TESTBENCH";
+    
     // Memory methods
     tab(n, *fOut);
     if (!gGlobal->gLightMode) {
@@ -558,6 +560,9 @@ void CScalarOneSampleCodeContainer::produceClass()
     tab(n, *fOut);
     tab(n, *fOut);
     *fOut << "void instanceInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
+    // staticInit has to be called for each instance since the tables are actually not shared between instances
+    tab(n + 1, *fOut);
+    *fOut << "staticInit" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
     tab(n + 1, *fOut);
     *fOut << "instanceConstants" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
     tab(n + 1, *fOut);
@@ -571,8 +576,6 @@ void CScalarOneSampleCodeContainer::produceClass()
     tab(n, *fOut);
     *fOut << "void init" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
     tab(n + 1, *fOut);
-    *fOut << "staticInit" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
-    tab(n + 1, *fOut);
     *fOut << "instanceInit" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
     tab(n, *fOut);
     *fOut << "}";
@@ -590,7 +593,7 @@ void CScalarOneSampleCodeContainer::produceClass()
     }
     
     tab(n, *fOut);
-    *fOut << "void control" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* iControl, $0* fControl, int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void control" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
     tab(n + 1, *fOut);
     fCodeProducer->Tab(n + 1);
     // Generates local variables declaration and setup
@@ -616,6 +619,9 @@ void CScalarOneSampleCodeContainer::produceClass()
     
     // Compute
     generateCompute(n);
+    
+    tab(n, *fOut);
+    *fOut << "#endif // TESTBENCH" << endl;
     tab(n, *fOut);
     
     // Generate user interface macros if needed
@@ -665,11 +671,10 @@ CScalarCodeContainer::CScalarCodeContainer(const string& name, int numInputs, in
     fSubContainerType = sub_container_type;
 }
 
-void CScalarCodeContainer::generateCompute(int n)
+void CScalarCodeContainer::generateComputeAux(int n)
 {
     // Generates declaration
     tab(n, *fOut);
-    tab(n, *fOut);
     *fOut << "void compute" << fKlassName << "(" << fKlassName
           << subst("* dsp, int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
     tab(n + 1, *fOut);
@@ -683,22 +688,27 @@ void CScalarCodeContainer::generateCompute(int n)
     loop->accept(fCodeProducer);
   
     /*
-    // TODO : atomic switch
-    // Currently for soundfile management
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
     generatePostComputeBlock(fCodeProducer);
-    */
-
+    
     back(1, *fOut);
     *fOut << "}" << endl;
 }
 
-void CScalarOneSampleCodeContainer::generateCompute(int n)
+// Special version for -os generation mode
+void CScalarOneSampleCodeContainer::generateComputeAux(int n)
 {
     // Generates declaration
     tab(n, *fOut);
-    tab(n, *fOut);
-    *fOut << "void compute" << fKlassName << "(" << fKlassName
-          << subst("* dsp, $0* inputs, $0* outputs, int* iControl, $0* fControl, int* iZone, $0* fZone) {", xfloat());
+    if (gGlobal->gInPlace) {
+        *fOut << "void compute" << fKlassName << "(" << fKlassName
+              << subst("* dsp, $0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
+    } else {
+        *fOut << "void compute" << fKlassName << "(" << fKlassName
+              << subst("* dsp, $0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
+    }
     tab(n + 1, *fOut);
     fCodeProducer->Tab(n + 1);
     
@@ -709,8 +719,8 @@ void CScalarOneSampleCodeContainer::generateCompute(int n)
     /*
      // TODO : atomic switch
      // Currently for soundfile management
-     generatePostComputeBlock(fCodeProducer);
      */
+    generatePostComputeBlock(fCodeProducer);
     
     back(1, *fOut);
     *fOut << "}" << endl;
@@ -722,13 +732,8 @@ CVectorCodeContainer::CVectorCodeContainer(const string& name, int numInputs, in
 {
 }
 
-void CVectorCodeContainer::generateCompute(int n)
+void CVectorCodeContainer::generateComputeAux(int n)
 {
-    // Possibly generate separated functions
-    fCodeProducer->Tab(n);
-    tab(n, *fOut);
-    generateComputeFunctions(fCodeProducer);
-
     // Generates declaration
     tab(n, *fOut);
     *fOut << "void compute" << fKlassName << "(" << fKlassName
@@ -752,13 +757,8 @@ COpenMPCodeContainer::COpenMPCodeContainer(const string& name, int numInputs, in
 {
 }
 
-void COpenMPCodeContainer::generateCompute(int n)
+void COpenMPCodeContainer::generateComputeAux(int n)
 {
-    // Possibly generate separated functions
-    fCodeProducer->Tab(n);
-    tab(n, *fOut);
-    generateComputeFunctions(fCodeProducer);
-
     // Compute declaration
     tab(n, *fOut);
     *fOut << "void compute" << fKlassName << "(" << fKlassName
@@ -783,13 +783,8 @@ CWorkStealingCodeContainer::CWorkStealingCodeContainer(const string& name, int n
 {
 }
 
-void CWorkStealingCodeContainer::generateCompute(int n)
+void CWorkStealingCodeContainer::generateComputeAux(int n)
 {
-    // Possibly generate separated functions
-    fCodeProducer->Tab(n);
-    tab(n, *fOut);
-    generateComputeFunctions(fCodeProducer);
-
     // Generates "computeThread" code
     tab(n, *fOut);
     *fOut << "static void computeThread" << fKlassName << "(" << fKlassName << "* dsp, int num_thread) {";
