@@ -46,6 +46,8 @@ CodeContainer::CodeContainer()
       fNumOutputs(-1),
       fNumActives(0),
       fNumPassives(0),
+      fSubContainerType(kInt),
+      fGeneratedSR(false),
       fExtGlobalDeclarationInstructions(InstBuilder::genBlockInst()),
       fGlobalDeclarationInstructions(InstBuilder::genBlockInst()),
       fDeclarationInstructions(InstBuilder::genBlockInst()),
@@ -62,8 +64,6 @@ CodeContainer::CodeContainer()
       fPostComputeBlockInstructions(InstBuilder::genBlockInst()),
       fComputeFunctions(InstBuilder::genBlockInst()),
       fUserInterfaceInstructions(InstBuilder::genBlockInst()),
-      fSubContainerType(kInt),
-      fGeneratedSR(false),
       fInt32ControlNum(0),
       fRealControlNum(0)
 {
@@ -71,8 +71,7 @@ CodeContainer::CodeContainer()
 }
 
 CodeContainer::~CodeContainer()
-{
-}
+{}
 
 void CodeContainer::transformDAG(DispatchVisitor* visitor)
 {
@@ -102,6 +101,24 @@ bool CodeContainer::getLoopProperty(Tree sig, CodeLoop*& l)
     return fLoopProperty.get(sig, l);
 }
 
+void CodeContainer::listAllLoopProperties(Tree sig, set<CodeLoop*>& L, set<Tree>& visited)
+{
+    if (visited.count(sig) == 0) {
+        visited.insert(sig);
+        CodeLoop* l;
+        if (getLoopProperty(sig, l)) {
+            L.insert(l);
+        } else {
+            // we go down the expression
+            vector<Tree> subsigs;
+            int          n = getSubSignals(sig, subsigs, false);
+            for (int i = 0; i < n; i++) {
+                listAllLoopProperties(subsigs[i], L, visited);
+            }
+        }
+    }
+}
+
 /**
  * Open a non-recursive loop on top of the stack of open loops.
  * @param size the number of iterations of the loop
@@ -128,6 +145,15 @@ void CodeContainer::openLoop(Tree recsymbol, const string& index_name, int size)
 void CodeContainer::closeLoop(Tree sig)
 {
     faustassert(fCurLoop);
+    
+    // fix the missing dependencies
+    set<CodeLoop*> L;
+    set<Tree> V;
+    listAllLoopProperties(sig, L, V);
+    for (CodeLoop* l : L) {
+        fCurLoop->fBackwardLoopDependencies.insert(l);
+    }
+    
     CodeLoop* l = fCurLoop;
     fCurLoop    = l->fEnclosingLoop;
     faustassert(fCurLoop);
@@ -250,7 +276,6 @@ void CodeContainer::computeForwardDAG(lclgraph dag, int& loop_count, vector<int>
 ValueInst* CodeContainer::pushFunction(const string& name, Typed::VarType result, vector<Typed::VarType>& types,
                                        const list<ValueInst*>& args)
 {
-    list<ValueInst*>::const_iterator it = args.begin();
     list<NamedTyped*> named_args;
     for (size_t i = 0; i < types.size(); i++) {
         named_args.push_back(InstBuilder::genNamedTyped("dummy" + to_string(i), InstBuilder::genBasicTyped(types[i])));
@@ -408,18 +433,21 @@ BlockInst* CodeContainer::inlineSubcontainersFunCalls(BlockInst* block)
 {
     // Rename 'sig' in 'dsp' and remove 'dsp' allocation
     block = DspRenamer().getCode(block);
+    //dump2FIR(block);
 
     // Inline subcontainers 'instanceInit' and 'fill' function call
     for (auto& it : fSubContainers) {
         // Build the function to be inlined (prototype and code)
         DeclareFunInst* inst_init_fun = it->generateInstanceInitFun("instanceInit" + it->getClassName(), "dsp", true, false);
-        // dump2FIR(inst_init_fun);
+        //dump2FIR(inst_init_fun);
         block = FunctionCallInliner(inst_init_fun).getCode(block);
-
+        //dump2FIR(block);
+    
         // Build the function to be inlined (prototype and code)
         DeclareFunInst* fill_fun = it->generateFillFun("fill" + it->getClassName(), "dsp", true, false);
-        // dump2FIR(fill_fun);
+        //dump2FIR(fill_fun);
         block = FunctionCallInliner(fill_fun).getCode(block);
+        //dump2FIR(block);
     }
 
     // dump2FIR(block);
@@ -433,6 +461,16 @@ void CodeContainer::printMacros(ostream& fout, int n)
         tab(n, fout);
         fout << "#ifdef FAUST_UIMACROS";
         tab(n + 1, fout);
+        tab(n + 1, fout);
+        for (auto& it : gGlobal->gMetaDataSet) {
+            if (it.first == tree("filename")) {
+                fout << "#define FAUST_FILE_NAME " << **(it.second.begin());
+                break;
+            }
+        }
+        tab(n + 1, fout);
+        fout << "#define FAUST_CLASS_NAME " << "\"" << fKlassName << "\"";
+        tab(n + 1, fout);
         fout << "#define FAUST_INPUTS " << fNumInputs;
         tab(n + 1, fout);
         fout << "#define FAUST_OUTPUTS " << fNumOutputs;
@@ -440,9 +478,23 @@ void CodeContainer::printMacros(ostream& fout, int n)
         fout << "#define FAUST_ACTIVES " << fNumActives;
         tab(n + 1, fout);
         fout << "#define FAUST_PASSIVES " << fNumPassives;
+        tab(n, fout);
         printlines(n + 1, fUIMacro, fout);
         tab(n, fout);
-        fout << "#endif";
+        tab(n, fout);
+        {
+            fout << "\t" << "#define FAUST_LIST_ACTIVES(p) \\";
+            printlines(n + 2, fUIMacroActives, fout);
+            tab(n, fout);
+            tab(n, fout);
+        }
+        {
+            fout << "\t" << "#define FAUST_LIST_PASSIVES(p) \\";
+            printlines(n + 2, fUIMacroPassives, fout);
+            tab(n, fout);
+            tab(n, fout);
+        }
+        fout << "#endif" << endl;
     }
 }
 
@@ -676,21 +728,21 @@ DeclareFunInst* CodeContainer::generateInit(const string& name, const string& ob
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args;
+        list<ValueInst*> args1;
         if (!ismethod) {
-            args.push_back(InstBuilder::genLoadFunArgsVar(obj));
+            args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
-        args.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
-        block->pushBackInst(InstBuilder::genVoidFunCallInst("classInit", args));
+        args1.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
+        block->pushBackInst(InstBuilder::genVoidFunCallInst("classInit", args1));
     }
 
     {
-        list<ValueInst*> args;
+        list<ValueInst*> args1;
         if (!ismethod) {
-            args.push_back(InstBuilder::genLoadFunArgsVar(obj));
+            args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
-        args.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
-        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceInit", args));
+        args1.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
+        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceInit", args1));
     }
 
     // Creates function
@@ -708,28 +760,28 @@ DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const st
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args;
+        list<ValueInst*> args1;
         if (!ismethod) {
-            args.push_back(InstBuilder::genLoadFunArgsVar(obj));
+            args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
-        args.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
-        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceConstants", args));
+        args1.push_back(InstBuilder::genLoadFunArgsVar("sample_rate"));
+        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceConstants", args1));
     }
 
     {
-        list<ValueInst*> args;
+        list<ValueInst*> args1;
         if (!ismethod) {
-            args.push_back(InstBuilder::genLoadFunArgsVar(obj));
+            args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
-        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceResetUserInterface", args));
+        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceResetUserInterface", args1));
     }
 
     {
-        list<ValueInst*> args;
+        list<ValueInst*> args1;
         if (!ismethod) {
-            args.push_back(InstBuilder::genLoadFunArgsVar(obj));
+            args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
-        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceClear", args));
+        block->pushBackInst(InstBuilder::genVoidFunCallInst("instanceClear", args1));
     }
 
     // Creates function
@@ -803,11 +855,11 @@ DeclareFunInst* CodeContainer::generateNewDsp(const string& name, int size)
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args;
-        args.push_back(InstBuilder::genInt64NumInst(1));
-        args.push_back(InstBuilder::genInt64NumInst(size));
+        list<ValueInst*> args1;
+        args1.push_back(InstBuilder::genInt64NumInst(1));
+        args1.push_back(InstBuilder::genInt64NumInst(size));
         block->pushBackInst(InstBuilder::genRetInst(InstBuilder::genCastInst(
-            InstBuilder::genFunCallInst("calloc", args), InstBuilder::genBasicTyped(Typed::kObj_ptr))));
+            InstBuilder::genFunCallInst("calloc", args1), InstBuilder::genBasicTyped(Typed::kObj_ptr))));
     }
 
     // Creates function
@@ -822,10 +874,10 @@ DeclareFunInst* CodeContainer::generateDeleteDsp(const string& name, const strin
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args;
-        args.push_back(InstBuilder::genCastInst(InstBuilder::genLoadFunArgsVar(obj),
+        list<ValueInst*> args1;
+        args1.push_back(InstBuilder::genCastInst(InstBuilder::genLoadFunArgsVar(obj),
                                                 InstBuilder::genBasicTyped(Typed::kVoid_ptr)));
-        block->pushBackInst(InstBuilder::genDropInst(InstBuilder::genFunCallInst("free", args)));
+        block->pushBackInst(InstBuilder::genDropInst(InstBuilder::genFunCallInst("free", args1)));
     }
 
     // Explicit return
