@@ -87,28 +87,28 @@ string VectorCompiler::CS(Tree sig)
 
         if (fClass->getLoopProperty(sig, ls)) {
             // sig has a loop property
-            // cerr << "CASE SH : fBackwardLoopDependencies.insert : " << tl << " --depend(A)son--> " << ls << endl;
+            //cerr << "CASE SH : fBackwardLoopDependencies.insert : " << tl << " --depend(A)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
         } else if (isSigFixDelay(sig, x, d) && fClass->getLoopProperty(x, ls)) {
-            // cerr << "CASE DL : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
+            //cerr << "CASE DL : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
         } else if (isSigFixDelay(sig, x, d) && isProj(x, &i, r) && fClass->getLoopProperty(r, ls)) {
-            // cerr << "CASE DR : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
+            //cerr << "CASE DR : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
         } else if (isProj(sig, &i, r) && fClass->getLoopProperty(r, ls)) {
-            // cerr << "CASE R* : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
+            //cerr << "CASE R* : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
         } else {
             if (isProj(sig, &i, r)) {
-                // cerr << "SYMBOL RECURSIF EN COURS ??? " << *r << endl;
+                //cerr << "SYMBOL RECURSIF EN COURS ??? " << *r << endl;
             } else if (getCertifiedSigType(sig)->variability() < kSamp) {
-                // cerr << "SLOW EXPRESSION " << endl;
+                //cerr << "SLOW EXPRESSION " << endl;
             } else {
-                // cerr << "Expression absorbée" << *sig << endl;
+                //cerr << "Expression absorbée" << *sig << endl;
             }
         }
     }
@@ -246,19 +246,27 @@ string VectorCompiler::generateCacheCode(Tree sig, const string& exp)
         // sample-rate signal
         if (d > 0) {
             // used delayed : we need a delay line
+            //cerr << "CHASING BUG 1 " << *sig << endl;
+            //cerr << "CHASING BUG T " << getCertifiedSigType(sig) << endl;
             getTypedNames(getCertifiedSigType(sig), "Yec", ctype, vname);
+            //cerr << "CHASING BUG N " << ctype << " " << vname << endl;
             generateDelayLine(ctype, vname, d, exp, getConditionCode(sig));
             setVectorNameProperty(sig, vname);
 
             if (verySimple(sig)) {
+                //cerr << "CHASING BUG 2 " << exp << endl;
                 return exp;
             } else {
                 if (d < gGlobal->gMaxCopyDelay) {
-                    return subst("$0[i]", vname);
+                    string sss = subst("$0[i]", vname);
+                    //cerr << "CHASING BUG 3 " << sss << endl;
+                    return sss;
                 } else {
                     // we use a ring buffer
                     string mask = T(pow2limit(d + gGlobal->gVecSize) - 1);
-                    return subst("$0[($0_idx+i) & $1]", vname, mask);
+                    string sss  = subst("$0[($0_idx+i) & $1]", vname, mask);
+                    //cerr << "CHASING BUG 4 " << sss << endl;
+                    return sss;
                 }
             }
         } else {
@@ -315,11 +323,13 @@ bool VectorCompiler::needSeparateLoop(Tree sig)
         // and not shared, doesn't require a separate loop.
         b = false;
     }
-    /*    if (b) {
-            cerr << "Separate Loop for " << ppsig(sig) << endl;
-        } else {
-            cerr << "Same Loop for " << ppsig(sig) << endl;
-        }*/
+    /*
+    if (b) {
+        cerr << "Separate Loop for " << ppsig(sig) << endl;
+    } else {
+        cerr << "Same Loop for " << ppsig(sig) << endl;
+    }
+    */
     return b;
 }
 
@@ -339,7 +349,7 @@ string VectorCompiler::generateVariableStore(Tree sig, const string& exp)
 
 /**
  * Generate code for accessing a delayed signal. The generated code depend of
- * the maximum delay attached to exp and the gLessTempSwitch.
+ * the maximum delay attached to exp.
  */
 
 string VectorCompiler::generateFixDelay(Tree sig, Tree exp, Tree delay)
@@ -419,15 +429,6 @@ void VectorCompiler::generateDelayLine(const string& ctype, const string& vname,
     }
 }
 
-#if 0
-static int pow2limit(int x)
-{
-    int n = 2;
-    while (n < x) { n = 2*n; }
-    return n;
-}
-#endif
-
 /**
  * Generate the code for a (short) delay line
  * @param k the c++ class where the delay line will be placed.
