@@ -531,10 +531,10 @@ void CPPScalarCodeContainer::generateCompute(int n)
     loop->accept(&fCodeProducer);
    
     /*
-    // TODO : atomic switch
-    // Currently for soundfile management
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
     generatePostComputeBlock(&fCodeProducer);
-    */
 
     back(1, *fOut);
     *fOut << "}";
@@ -547,7 +547,11 @@ void CPPScalarOneSampleCodeContainer::generateCompute(int n)
     // Generates declaration
     tab(n + 1, *fOut);
     tab(n + 1, *fOut);
-    *fOut << subst("virtual void compute($0* inputs, $0* outputs, int* iControl, $0* fControl) {", xfloat());
+    if (gGlobal->gInPlace) {
+        *fOut << subst("virtual void compute($0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    } else {
+        *fOut << subst("virtual void compute($0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    }
     tab(n + 2, *fOut);
     fCodeProducer.Tab(n + 2);
     
@@ -558,8 +562,8 @@ void CPPScalarOneSampleCodeContainer::generateCompute(int n)
     /*
      // TODO : atomic switch
      // Currently for soundfile management
-     generatePostComputeBlock(&fCodeProducer);
      */
+    generatePostComputeBlock(&fCodeProducer);
     
     back(1, *fOut);
     *fOut << "}";
@@ -598,6 +602,13 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     *fOut << "#endif" << endl;
     tab(n, *fOut);
     
+    *fOut << "#if defined(_WIN32)" << endl;
+    *fOut << "#define RESTRICT __restrict" << endl;
+    *fOut << "#else" << endl;
+    *fOut << "#define RESTRICT __restrict__" << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+    
     *fOut << "#define FAUST_INPUTS " << fNumInputs << endl;
     *fOut << "#define FAUST_OUTPUTS  " << fNumOutputs << endl;
     *fOut << "#define FAUST_INT_CONTROLS " << fInt32ControlNum  << endl;
@@ -751,7 +762,7 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     *fOut << "}";
     
     tab(n + 1, *fOut);
-    *fOut << subst("virtual void control(int* iControl, $0* fControl) {", xfloat());
+    *fOut << subst("virtual void control(int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
     tab(n + 2, *fOut);
     fCodeProducer.Tab(n + 2);
     // Generates local variables declaration and setup
