@@ -114,7 +114,13 @@ void DAGInstructionsCompiler::compileMultiSignal(Tree L)
 
             // Cast to external float
             ValueInst* res = InstBuilder::genCastFloatMacroInst(CS(sig));
-            pushComputeDSPMethod(InstBuilder::genStoreArrayStackVar(name, getCurrentLoopIndex(), res));
+            
+            if (gGlobal->gComputeMix) {
+                ValueInst* res1 = InstBuilder::genAdd(res, InstBuilder::genLoadArrayStackVar(name, getCurrentLoopIndex()));
+                pushComputeDSPMethod(InstBuilder::genStoreArrayStackVar(name, getCurrentLoopIndex(), res1));
+            } else {
+                pushComputeDSPMethod(InstBuilder::genStoreArrayStackVar(name, getCurrentLoopIndex(), res));
+            }
 
             fContainer->closeLoop(sig);
         }
@@ -286,7 +292,7 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
     Typed::VarType ctype;
     int            sharing = getSharingCount(sig);
     ::Type         t       = getCertifiedSigType(sig);
-    Occurences*    o       = fOccMarkup.retrieve(sig);
+    old_Occurences*    o   = fOccMarkup->retrieve(sig);
     int            d       = o->getMaxDelay();
 
     if (t->variability() < kSamp) {
@@ -304,13 +310,13 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
                 // first cache this expression because it
                 // it is shared and complex
                 ValueInst* cachedexp = generateVariableStore(sig, exp);
-                generateDelayLine(cachedexp, ctype, vname, d, var_access);
+                generateDelayLine(cachedexp, ctype, vname, d, var_access, nullptr);
                 setVectorNameProperty(sig, vname);
                 return cachedexp;
             } else {
                 // no need to cache this expression because
                 // it is either not shared or very simple
-                generateDelayLine(exp, ctype, vname, d, var_access);
+                generateDelayLine(exp, ctype, vname, d, var_access, nullptr);
                 setVectorNameProperty(sig, vname);
                 return exp;
             }
@@ -321,7 +327,7 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
             // used delayed : we need a delay line
             getTypedNames(getCertifiedSigType(sig), "Yec", ctype, vname);
             Address::AccessType var_access;
-            generateDelayLine(exp, ctype, vname, d, var_access);
+            generateDelayLine(exp, ctype, vname, d, var_access, nullptr);
             setVectorNameProperty(sig, vname);
 
             if (verySimple(sig)) {
@@ -335,8 +341,7 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
                     string vname_idx = vname + "_idx";
                     int    mask      = pow2limit(d + gGlobal->gVecSize) - 1;
                     // return subst("$0[($0_idx+i) & $1]", vname, mask);
-                    FIRIndex index1 = (getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx)) &
-                                      InstBuilder::genInt32NumInst(mask);
+                    FIRIndex index1 = (getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx)) & mask;
                     return InstBuilder::genLoadArrayStructVar(vname, index1);
                 }
             }
@@ -351,7 +356,7 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
                 // cerr << "Zec : " << ppsig(sig) << endl;
                 getTypedNames(getCertifiedSigType(sig), "Zec", ctype, vname);
                 Address::AccessType var_access;
-                generateDelayLine(exp, ctype, vname, d, var_access);
+                generateDelayLine(exp, ctype, vname, d, var_access, nullptr);
                 setVectorNameProperty(sig, vname);
                 // return subst("$0[i]", vname);
                 return InstBuilder::genLoadArrayVar(vname, var_access, getCurrentLoopIndex());
@@ -372,7 +377,7 @@ ValueInst* DAGInstructionsCompiler::generateCacheCode(Tree sig, ValueInst* exp)
  */
 bool DAGInstructionsCompiler::needSeparateLoop(Tree sig)
 {
-    Occurences* o = fOccMarkup.retrieve(sig);
+    old_Occurences* o = fOccMarkup->retrieve(sig);
     ::Type      t = getCertifiedSigType(sig);
     int         c = getSharingCount(sig);
     bool        b;
@@ -439,7 +444,7 @@ ValueInst* DAGInstructionsCompiler::generateFixDelay(Tree sig, Tree exp, Tree de
 {
     string     vname;
     ValueInst* code = CS(exp);  // ensure exp is compiled to have a vector name
-    int        d, mxd = fOccMarkup.retrieve(exp)->getMaxDelay();
+    int        d, mxd = fOccMarkup->retrieve(exp)->getMaxDelay();
 
     if (!getVectorNameProperty(exp, vname)) {
         if (mxd == 0) {
@@ -464,7 +469,7 @@ ValueInst* DAGInstructionsCompiler::generateFixDelay(Tree sig, Tree exp, Tree de
                 return InstBuilder::genLoadArrayStackVar(vname, getCurrentLoopIndex());
             } else {
                 // return subst("$0[i-$1]", vname, T(d));
-                FIRIndex index = getCurrentLoopIndex() - InstBuilder::genInt32NumInst(d);
+                FIRIndex index = getCurrentLoopIndex() - d;
                 return generateCacheCode(sig, InstBuilder::genLoadArrayStackVar(vname, index));
             }
         } else {
@@ -480,21 +485,20 @@ ValueInst* DAGInstructionsCompiler::generateFixDelay(Tree sig, Tree exp, Tree de
         if (isSigInt(delay, &d)) {
             if (d == 0) {
                 // return subst("$0[($0_idx+i)&$1]", vname, T(N-1));
-                FIRIndex index1 = (getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx)) &
-                                  InstBuilder::genInt32NumInst(N - 1);
+                FIRIndex index1 = (getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx)) & (N - 1);
                 return generateCacheCode(sig, InstBuilder::genLoadArrayStructVar(vname, index1));
             } else {
                 // return subst("$0[($0_idx+i-$2)&$1]", vname, T(N-1), T(d));
                 FIRIndex index1 = getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx);
-                FIRIndex index2 = index1 - InstBuilder::genInt32NumInst(d);
-                FIRIndex index3 = index2 & InstBuilder::genInt32NumInst(N - 1);
+                FIRIndex index2 = index1 - d;
+                FIRIndex index3 = index2 & (N - 1);
                 return generateCacheCode(sig, InstBuilder::genLoadArrayStructVar(vname, index3));
             }
         } else {
             // return subst("$0[($0_idx+i-$2)&$1]", vname, T(N-1), CS(delay));
             FIRIndex index1 = getCurrentLoopIndex() + InstBuilder::genLoadStructVar(vname_idx);
             FIRIndex index2 = index1 - CS(delay);
-            FIRIndex index3 = index2 & InstBuilder::genInt32NumInst(N - 1);
+            FIRIndex index3 = index2 & (N - 1);
             return generateCacheCode(sig, InstBuilder::genLoadArrayStructVar(vname, index3));
         }
     }
@@ -508,7 +512,7 @@ ValueInst* DAGInstructionsCompiler::generateDelayVec(Tree sig, ValueInst* exp, T
 
     setVectorNameProperty(sig, vname);
     Address::AccessType var_access;
-    generateDelayLine(exp, ctype, vname, mxd, var_access);
+    generateDelayLine(exp, ctype, vname, mxd, var_access, nullptr);
 
     if (verySimple(sig)) {
         return exp;
@@ -518,7 +522,7 @@ ValueInst* DAGInstructionsCompiler::generateDelayVec(Tree sig, ValueInst* exp, T
 }
 
 ValueInst* DAGInstructionsCompiler::generateDelayLine(ValueInst* exp, Typed::VarType ctype, const string& vname,
-                                                      int mxd, Address::AccessType& var_access)
+                                                      int mxd, Address::AccessType& var_access, ValueInst* unused)
 {
     if (mxd == 0) {
         generateVectorLoop(ctype, vname, exp, var_access);
@@ -575,13 +579,13 @@ void DAGInstructionsCompiler::generateDlineLoop(Typed::VarType ctype, const stri
         pushComputeBlockMethod(table_inst2);
 
         // -- copy the stored samples to the delay line
-        pushComputePreDSPMethod(generateCopyArray(buf, pmem, delay));
+        pushPreComputeDSPMethod(generateCopyArray(buf, pmem, delay));
 
         // -- compute the new samples
         pushComputeDSPMethod(InstBuilder::genStoreArrayStackVar(vname, getCurrentLoopIndex(), exp));
 
         // -- copy back to stored samples
-        pushComputePostDSPMethod(generateCopyBackArray(pmem, buf, delay));
+        pushPostComputeDSPMethod(generateCopyBackArray(pmem, buf, delay));
 
         // Set desired variable access
         var_access = Address::kStack;
@@ -605,18 +609,18 @@ void DAGInstructionsCompiler::generateDlineLoop(Typed::VarType ctype, const stri
 
         // -- update index
         FIRIndex index1 = FIRIndex(InstBuilder::genLoadStructVar(idx)) + InstBuilder::genLoadStructVar(idx_save);
-        FIRIndex index2 = index1 & InstBuilder::genInt32NumInst(delay - 1);
+        FIRIndex index2 = index1 & (delay - 1);
 
-        pushComputePreDSPMethod(InstBuilder::genStoreStructVar(idx, index2));
+        pushPreComputeDSPMethod(InstBuilder::genStoreStructVar(idx, index2));
 
         // -- compute the new samples
         FIRIndex index3 = getCurrentLoopIndex() + InstBuilder::genLoadStructVar(idx);
-        FIRIndex index4 = index3 & InstBuilder::genInt32NumInst(delay - 1);
+        FIRIndex index4 = index3 & (delay - 1);
 
         pushComputeDSPMethod(InstBuilder::genStoreArrayStructVar(vname, index4, exp));
 
         // -- save index
-        pushComputePostDSPMethod(InstBuilder::genStoreStructVar(idx_save, InstBuilder::genLoadLoopVar("vsize")));
+        pushPostComputeDSPMethod(InstBuilder::genStoreStructVar(idx_save, InstBuilder::genLoadLoopVar("vsize")));
 
         // Set desired variable access
         var_access = Address::kStruct;
@@ -651,10 +655,8 @@ ValueInst* DAGInstructionsCompiler::generateWaveform(Tree sig)
     declareWaveform(sig, vname, size);
 
     string   idx    = subst("$0_idx", vname);
-    FIRIndex index1 = (FIRIndex(InstBuilder::genLoadStructVar(idx)) + InstBuilder::genLoadLoopVar("vsize")) %
-                      InstBuilder::genInt32NumInst(size);
-    pushComputePostDSPMethod(InstBuilder::genStoreStructVar(idx, index1));
-    FIRIndex index2 =
-        (FIRIndex(InstBuilder::genLoadStructVar(idx)) + getCurrentLoopIndex()) % InstBuilder::genInt32NumInst(size);
+    FIRIndex index1 = (FIRIndex(InstBuilder::genLoadStructVar(idx)) + InstBuilder::genLoadLoopVar("vsize")) % size;
+    pushPostComputeDSPMethod(InstBuilder::genStoreStructVar(idx, index1));
+    FIRIndex index2 = (FIRIndex(InstBuilder::genLoadStructVar(idx)) + getCurrentLoopIndex()) % size;
     return generateCacheCode(sig, InstBuilder::genLoadArrayStaticStructVar(vname, index2));
 }
