@@ -148,13 +148,6 @@ DeclareFunInst* WASTCodeContainer::generateInstanceInitFun(const string& name, c
     return InstBuilder::genVoidFunction(name, args, init_block, isvirtual);
 }
 
-void WASTCodeContainer::produceInternal()
-{
-    // Fields generation
-    generateGlobalDeclarations(gGlobal->gWASTVisitor);
-    generateDeclarations(gGlobal->gWASTVisitor);
-}
-
 void WASTCodeContainer::produceClass()
 {
     int n = 0;
@@ -207,9 +200,6 @@ void WASTCodeContainer::produceClass()
     gGlobal->gWASTVisitor->Tab(n + 1);
     generateDeclarations(gGlobal->gWASTVisitor);
 
-    // After field declaration
-    generateSubContainers();
-
     // Keep location of memory generation
     streampos begin_memory = fOutAux.tellp();
 
@@ -457,6 +447,9 @@ void WASTScalarCodeContainer::generateCompute(int n)
     // Loop 'i' variable is moved by bytes
     BlockInst* compute_block = InstBuilder::genBlockInst();
     compute_block->pushBackInst(fCurLoop->generateScalarLoop(fFullCount, gGlobal->gLoopVarInBytes));
+    
+    // Generates post DSP loop code
+    compute_block->pushBackInst(fPostComputeBlockInstructions);
 
     generateComputeAux2(compute_block, n);
 }
