@@ -78,6 +78,10 @@
 #include "rust_code_container.hh"
 #endif
 
+#ifdef DLANG_BUILD
+#include "dlang_code_container.hh"
+#endif
+
 // Parser
 extern FILE*       yyin;
 extern const char* yyfilename;
@@ -114,7 +118,6 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gMaxNameSize      = 40;
     gSimpleNames      = false;
     gSimplifyDiagrams = false;
-    gLessTempSwitch   = false;
     gMaxCopyDelay     = 16;
 
     gVectorSwitch      = false;
@@ -133,6 +136,7 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gUIMacroSwitch = false;
     gDumpNorm      = false;
     gFTZMode       = 0;
+    gRangeUI       = false;
 
     gFloatSize = 1;
 
@@ -142,7 +146,7 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gDSPStruct  = false;
     gLightMode  = false;
     gClang      = false;
-    gCheckTable = false;
+    gCheckTable = "";
 
     gClassName      = "mydsp";
     gSuperClassName = "dsp";
@@ -154,8 +158,10 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
 
     // Backend configuration : default values
     gAllowForeignFunction = true;
+    gAllowForeignConstant = true;
+    gAllowForeignVar      = true;
     gComputeIOTA          = false;
-    gFAUSTFLOATToInternal = false;
+    gFAUSTFLOAT2Internal  = false;
     gInPlace              = false;
     gHasExp10             = false;
     gLoopVarInBytes       = false;
@@ -163,14 +169,17 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gUseDefaultSound      = true;
     gHasTeeLocal          = false;
     gFastMath             = false;
+    gMathApprox           = false;
     gNeedManualPow        = true;
     gRemoveVarAddress     = false;
     gOneSample            = false;
     gOneSampleControl     = false;
+    gComputeMix           = false;
     gFastMathLib          = "default";
     gNameSpace            = "";
 
     // Fastmath mapping float version
+    gFastMathLibTable["fabsf"]      = "fast_fabsf";
     gFastMathLibTable["acosf"]      = "fast_acosf";
     gFastMathLibTable["asinf"]      = "fast_asinf";
     gFastMathLibTable["atanf"]      = "fast_atanf";
@@ -187,12 +196,14 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gFastMathLibTable["log10f"]     = "fast_log10f";
     gFastMathLibTable["powf"]       = "fast_powf";
     gFastMathLibTable["remainderf"] = "fast_remainderf";
+    gFastMathLibTable["rintf"]      = "fast_rintf";
     gFastMathLibTable["roundf"]     = "fast_roundf";
     gFastMathLibTable["sinf"]       = "fast_sinf";
     gFastMathLibTable["sqrtf"]      = "fast_sqrtf";
     gFastMathLibTable["tanf"]       = "fast_tanf";
 
     // Fastmath mapping double version
+    gFastMathLibTable["fabs"]      = "fast_fabs";
     gFastMathLibTable["acos"]      = "fast_acos";
     gFastMathLibTable["asin"]      = "fast_asin";
     gFastMathLibTable["atan"]      = "fast_atan";
@@ -209,6 +220,7 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gFastMathLibTable["log10"]     = "fast_log10";
     gFastMathLibTable["pow"]       = "fast_pow";
     gFastMathLibTable["remainder"] = "fast_remainder";
+    gFastMathLibTable["rint"]      = "fast_rint";
     gFastMathLibTable["round"]     = "fast_round";
     gFastMathLibTable["sin"]       = "fast_sin";
     gFastMathLibTable["sqrt"]      = "fast_sqrt";
@@ -424,7 +436,6 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gPrintXMLSwitch   = false;
     gPrintJSONSwitch  = false;
     gPrintDocSwitch   = false;
-    gBalancedSwitch   = 0;
     gArchFile         = "";
     gExportDSP        = false;
 
@@ -453,6 +464,7 @@ void global::init()
     // Essential predefined types
     gMemoizedTypes   = new property<AudioType*>();
     gAllocationCount = 0;
+    gMaskDelayLineThreshold  = INT_MAX;
 
     // True by default but only usable with -lang ocpp backend
     gEnableFlag = true;
@@ -561,9 +573,17 @@ void global::init()
     gMathForeignFunctions["tanhf"] = true;
     gMathForeignFunctions["tanh"]  = true;
     gMathForeignFunctions["tanhl"] = true;
+    
+    gMathForeignFunctions["isnanf"] = true;
+    gMathForeignFunctions["isnan"] = true;
+    gMathForeignFunctions["isnanl"] = true;
+    
+    gMathForeignFunctions["isinff"] = true;
+    gMathForeignFunctions["isinf"] = true;
+    gMathForeignFunctions["isinfl"] = true;
 }
 
-void global::printCompilationOptions(ostream& dst, bool backend)
+void global::printCompilationOptions(stringstream& dst, bool backend)
 {
     if (backend) {
 #ifdef LLVM_BUILD
@@ -579,28 +599,29 @@ void global::printCompilationOptions(ostream& dst, bool backend)
     if (gInPlace) dst << "-inpl ";
     if (gOneSample) dst << "-os ";
     if (gLightMode) dst << "-light ";
-    if (gSchedulerSwitch) {
-        dst << "-sch"
-            << " -vs " << gVecSize << ((gFunTaskSwitch) ? " -fun" : "") << ((gGroupTaskSwitch) ? " -g" : "")
-            << ((gDeepFirstSwitch) ? " -dfs" : "")
-            << ((gFloatSize == 2) ? " -double" : (gFloatSize == 3) ? " -quad" : "") << " -ftz " << gFTZMode << " -mcd "
-            << gGlobal->gMaxCopyDelay << ((gMemoryManager) ? " -mem" : "");
-    } else if (gVectorSwitch) {
+    if (gMemoryManager) dst << "-mem ";
+    if (gComputeMix) dst << "-cm ";
+    if (gRangeUI) dst << "-rui ";
+    if (gMathApprox) dst << "-mapp ";
+    if (gMaskDelayLineThreshold != INT_MAX) dst << "-dtl " << gMaskDelayLineThreshold << " ";
+    dst << "-es " << gEnableFlag << " ";
+    if (gHasExp10) dst << "-exp10 ";
+    if (gOneSample) dst << "-os ";
+    if (gSchedulerSwitch) dst << "-sch ";
+    if (gOpenMPSwitch) dst << "-omp " << ((gOpenMPLoop) ? "-pl " : "");
+    if (gVectorSwitch) {
         dst << "-vec"
             << " -lv " << gVectorLoopVariant << " -vs " << gVecSize << ((gFunTaskSwitch) ? " -fun" : "")
             << ((gGroupTaskSwitch) ? " -g" : "") << ((gDeepFirstSwitch) ? " -dfs" : "")
             << ((gFloatSize == 2) ? " -double" : (gFloatSize == 3) ? " -quad" : "") << " -ftz " << gFTZMode << " -mcd "
-            << gGlobal->gMaxCopyDelay << ((gMemoryManager) ? " -mem" : "");
-    } else if (gOpenMPSwitch) {
-        dst << "-omp"
-            << " -vs " << gVecSize << " -vs " << gVecSize << ((gFunTaskSwitch) ? " -fun" : "")
-            << ((gGroupTaskSwitch) ? " -g" : "") << ((gDeepFirstSwitch) ? " -dfs" : "")
-            << ((gFloatSize == 2) ? " -double" : (gFloatSize == 3) ? " -quad" : "") << " -ftz " << gFTZMode << " -mcd "
-            << gGlobal->gMaxCopyDelay << ((gMemoryManager) ? " -mem" : "");
+            << gGlobal->gMaxCopyDelay;
     } else {
         dst << ((gFloatSize == 1) ? "-scal" : ((gFloatSize == 2) ? "-double" : (gFloatSize == 3) ? "-quad" : ""))
-            << " -ftz " << gFTZMode << ((gMemoryManager) ? " -mem" : "");
+            << " -ftz " << gFTZMode;
     }
+    
+    // Add 'compile_options' metadata
+    gGlobal->gMetaDataSet[tree("compile_options")].insert(tree("\"" + dst.str() + "\""));
 }
 
 void global::initTypeSizeMap()
@@ -675,6 +696,9 @@ global::~global()
 #ifdef RUST_BUILD
     RustInstVisitor::cleanup();
 #endif
+#ifdef DLANG_BUILD
+    DLangInstVisitor::cleanup();
+#endif
 }
 
 void global::allocate()
@@ -729,14 +753,6 @@ string global::getFreshID(const string& prefix)
     return subst("$0$1", prefix, T(n));
 }
 
-Garbageable::Garbageable()
-{
-}
-
-Garbageable::~Garbageable()
-{
-}
-
 void Garbageable::cleanup()
 {
     std::list<Garbageable*>::iterator it;
