@@ -1,6 +1,6 @@
 /************************************************************************
  FAUST Architecture File
- Copyright (C) 2012-2019 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2012-2020 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
  This Architecture section is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
@@ -54,11 +54,7 @@ struct MyMeta : public Meta, public std::map<std::string, std::string>
     }
     const std::string get(const char* key, const char* def)
     {
-        if (this->find(key) != this->end()) {
-            return (*this)[key];
-        } else {
-            return def;
-        }
+        return (this->find(key) != this->end()) ? (*this)[key] : def;
     }
 };
 
@@ -177,8 +173,6 @@ faustgen_factory::faustgen_factory(const string& name)
     fPolyphonic = false;
     fSoundUI = NULL;
     
-    fMidiHandler.startMidi();
-    
 #ifdef __APPLE__
     // OSX only : access to the fautgen~ bundle
     CFBundleRef faustgen_bundle = CFBundleGetBundleWithIdentifier(CFSTR("com.grame.faustgen-"));  // - character added since SDK 7.3.3
@@ -197,7 +191,7 @@ faustgen_factory::faustgen_factory(const string& name)
 #ifdef WIN32
     HMODULE handle = LoadLibrary("faustgen~.mxe64");
     if (handle) {
-        // Get faustgen~.mxe path
+        // Get faustgen~.mxe64 path
         char name[512];
         GetModuleFileName(handle, name, 512);
         string str_name = string(name);
@@ -221,11 +215,6 @@ faustgen_factory::faustgen_factory(const string& name)
         fDrawPath = "";
     }
 #endif
-    
-    t_max_err err = systhread_mutex_new(&fDSPMutex, SYSTHREAD_MUTEX_NORMAL);
-    if (err != MAX_ERR_NONE) {
-        post("Cannot allocate mutex...");
-    }
 }
 
 faustgen_factory::~faustgen_factory()
@@ -234,11 +223,8 @@ faustgen_factory::~faustgen_factory()
     free_sourcecode();
     free_bitcode();
     
-    fMidiHandler.stopMidi();
-    
     remove_svg();
-    systhread_mutex_free(fDSPMutex);
-    
+
     delete fSoundUI;
     fSoundUI = NULL;
 }
@@ -263,7 +249,9 @@ void faustgen_factory::free_bitcode()
 
 void faustgen_factory::free_dsp_factory()
 {
-    if (lock()) {
+    lock_audio();
+    lock_ui();
+    {
         // Free all instances
         for (auto& it : fInstances) {
             it->free_dsp();
@@ -271,10 +259,9 @@ void faustgen_factory::free_dsp_factory()
         
         //deleteDSPFactory(fDSPfactory);
         fDSPfactory = NULL;
-        unlock();
-    } else {
-        post("Mutex lock cannot be taken...");
     }
+    unlock_ui();
+    unlock_audio();
 }
 
 llvm_dsp_factory* faustgen_factory::create_factory_from_bitcode()
@@ -1051,6 +1038,7 @@ faustgen::faustgen(t_symbol* sym, long ac, t_atom* argv)
     fDSPUI = NULL;
     fMidiUI = NULL;
     fOSCUI = NULL;
+    fSavedUI = NULL;
     fDSPfactory = NULL;
     fEditor = NULL;
     fMute = false;
@@ -1059,6 +1047,8 @@ faustgen::faustgen(t_symbol* sym, long ac, t_atom* argv)
     t_atom* ap;
     bool res = false;
     
+    fMidiHandler.startMidi();
+    
     // Allocate factory with a given "name"
     for (i = 0, ap = argv; i < ac; i++, ap++) {
         if (atom_gettype(ap) == A_SYM) {
@@ -1084,7 +1074,7 @@ faustgen::faustgen(t_symbol* sym, long ac, t_atom* argv)
     
     // Needed to script objects
     char name[64];
-    sprintf(name, "faustgen-%x", this);
+    sprintf(name, "faustgen-%lld", (long long)this);
     jbox_set_varname(box, gensym(name));
     
     // Fetch the data inside the max patcher using the dictionary
@@ -1106,13 +1096,14 @@ faustgen::~faustgen()
         fEditor = NULL;
     }
     
-    fDSPfactory->remove_instance(this);
+    fDSPfactory->remove_instance(this);    
+    fMidiHandler.stopMidi();
 }
 
 void faustgen::free_dsp()
 {
     // Save controller state
-    fSavedUI.save();
+    fSavedUI->save();
     
     // Has to be done *before* remove_instance that may free fDSPfactory and thus fDSPfactory->fMidiHandler
     remove_midihandler();
@@ -1123,6 +1114,9 @@ void faustgen::free_dsp()
     delete fOSCUI;
     fOSCUI = NULL;
     
+    delete fSavedUI;
+    fSavedUI = NULL;
+    
     delete fDSPUI;
     fDSPUI = NULL;
     
@@ -1156,84 +1150,90 @@ t_dictionary* faustgen::json_reader(const char* jsontext)
 // Allows to set a value to the Faust module's parameter, or a list of values
 void faustgen::anything(long inlet, t_symbol* s, long ac, t_atom* av)
 {
-    bool res = false;
-    string name = string((s)->s_name);
-
-    // If no argument is there, consider it as a toggle message for a button
-    if (ac == 0 && fDSPUI->isValue(name)) {
+    fDSPfactory->lock_ui();
+    {
+        bool res = false;
+        string name = string((s)->s_name);
         
-        FAUSTFLOAT off = FAUSTFLOAT(0.0);
-        FAUSTFLOAT on = FAUSTFLOAT(1.0);
-        fDSPUI->setValue(name, off);
-        fDSPUI->setValue(name, on);
-        
-        av[0].a_type = A_FLOAT;
-        av[0].a_w.w_float = off;
-        anything(inlet, s, 1, av);
-        
-    } else if (mspUI::checkDigit(name)) { // List of values
-        
-        int ndigit = 0;
-        int pos;
-        
-        for (pos = name.size() - 1; pos >= 0; pos--) {
-            if (isdigit(name[pos]) || name[pos] == ' ') {
-                ndigit++;
-            } else {
-                break;
-            }
-        }
-        pos++;
-        
-        string prefix = name.substr(0, pos);
-        string num_base = name.substr(pos);
-        int num = atoi(num_base.c_str());
-        
-        int i;
-        t_atom* ap;
-        
-        // Increment ap each time to get to the next atom
-        for (i = 0, ap = av; i < ac; i++, ap++) {
-            FAUSTFLOAT value;
-            switch (atom_gettype(ap)) {
-                case A_LONG:
-                    value = FAUSTFLOAT(ap[0].a_w.w_long);
+        // If no argument is there, consider it as a toggle message for a button
+        if (ac == 0 && fDSPUI->isValue(name)) {
+            
+            FAUSTFLOAT off = FAUSTFLOAT(0.0);
+            FAUSTFLOAT on = FAUSTFLOAT(1.0);
+            fDSPUI->setValue(name, off);
+            fDSPUI->setValue(name, on);
+            
+            av[0].a_type = A_FLOAT;
+            av[0].a_w.w_float = off;
+            anything(inlet, s, 1, av);
+            
+            goto unlock;
+            
+        } else if (mspUI::checkDigit(name)) { // List of values
+            
+            int pos, ndigit = 0;
+            for (pos = name.size() - 1; pos >= 0; pos--) {
+                if (isdigit(name[pos]) || name[pos] == ' ') {
+                    ndigit++;
+                } else {
                     break;
-                case A_FLOAT:
-                    value = FAUSTFLOAT(ap[0].a_w.w_float);
-                    break;
-                default:
-                    post("Invalid argument in parameter setting");
-                    return;
+                }
+            }
+            pos++;
+            
+            string prefix = name.substr(0, pos);
+            string num_base = name.substr(pos);
+            int num = atoi(num_base.c_str());
+            
+            int i;
+            t_atom* ap;
+            
+            // Increment ap each time to get to the next atom
+            for (i = 0, ap = av; i < ac; i++, ap++) {
+                FAUSTFLOAT value;
+                switch (atom_gettype(ap)) {
+                    case A_LONG:
+                        value = FAUSTFLOAT(ap[0].a_w.w_long);
+                        break;
+                    case A_FLOAT:
+                        value = FAUSTFLOAT(ap[0].a_w.w_float);
+                        break;
+                    default:
+                        post("Invalid argument in parameter setting");
+                        goto unlock;
+                }
+                
+                string num_val = to_string(num + i);
+                stringstream param_name; param_name << prefix;
+                for (int i = 0; i < ndigit - mspUI::countDigit(num_val); i++) {
+                    param_name << ' ';
+                }
+                param_name << num_val;
+                
+                // Try special naming scheme for list of parameters
+                res = fDSPUI->setValue(param_name.str(), value);
+                
+                // Otherwise try standard name
+                if (!res) {
+                    res = fDSPUI->setValue(name, value);
+                }
+                if (!res) {
+                    post("Unknown parameter : %s", (s)->s_name);
+                }
             }
             
-            string num_val = to_string(num + i);
-            stringstream param_name; param_name << prefix;
-            for (int i = 0; i < ndigit - mspUI::countDigit(num_val); i++) {
-                param_name << ' ';
-            }
-            param_name << num_val;
-            
-            // Try special naming scheme for list of parameters
-            res = fDSPUI->setValue(param_name.str(), value);
-            
-            // Otherwise try standard name
-            if (!res) {
-                res = fDSPUI->setValue(name, value);
-            }
+        } else {
+            // Standard parameter name
+            FAUSTFLOAT value = (av[0].a_type == A_LONG) ? FAUSTFLOAT(av[0].a_w.w_long) : FAUSTFLOAT(av[0].a_w.w_float);
+            res = fDSPUI->setValue(name, value);
             if (!res) {
                 post("Unknown parameter : %s", (s)->s_name);
             }
         }
-        
-    } else {
-        // Standard parameter name
-        FAUSTFLOAT value = (av[0].a_type == A_LONG) ? FAUSTFLOAT(av[0].a_w.w_long) : av[0].a_w.w_float;
-        res = fDSPUI->setValue(name, value);
-        if (!res) {
-            post("Unknown parameter : %s", (s)->s_name);
-        }
     }
+        
+    unlock:
+        fDSPfactory->unlock_ui();
 }
 
 void faustgen::compileoptions(long inlet, t_symbol* s, long argc, t_atom* argv)
@@ -1253,7 +1253,9 @@ void faustgen::write(long inlet, t_symbol* s)
 
 void faustgen::polyphony(long inlet, t_symbol* s, long ac, t_atom* av)
 {
-    if (fDSPfactory->lock()) {
+    fDSPfactory->lock_audio();
+    fDSPfactory->lock_ui();
+    {
         free_dsp();
         fDSP = fDSPfactory->create_dsp_instance(av[0].a_w.w_long);
         assert(fDSP);
@@ -1269,67 +1271,76 @@ void faustgen::polyphony(long inlet, t_symbol* s, long ac, t_atom* av)
         
         // Initialize at the system's sampling rate
         fDSP->init(sys_getsr());
-        fDSPfactory->unlock();
-    } else {
-        post("Mutex lock cannot be taken...");
     }
+    fDSPfactory->unlock_audio();
+    fDSPfactory->unlock_ui();
 }
 
-// Reset controllers to init value and send [label, init, min, max]
+// Reset controllers to init value and send [path, init, min, max]
 void faustgen::init(long inlet, t_symbol* s, long ac, t_atom* av)
 {
     // Reset internal state
-    fSavedUI.reset();
+    fSavedUI->reset();
     
     // Input controllers
-    for (mspUI::iterator it = fDSPUI->begin1(); it != fDSPUI->end1(); it++) {
+    for (mspUI::iterator it = fDSPUI->begin2(); it != fDSPUI->end2(); it++) {
         t_atom myList[4];
         atom_setsym(&myList[0], gensym((*it).first.c_str()));
-        atom_setfloat(&myList[1], (*it).second->getInitValue());
+        atom_setfloat(&myList[1], (*it).second->getInitValue());    // init value
         atom_setfloat(&myList[2], (*it).second->getMinValue());
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(m_control_outlet, 0, 4, myList);
     }
     // Output controllers
-    for (mspUI::iterator it = fDSPUI->begin3(); it != fDSPUI->end3(); it++) {
+    for (mspUI::iterator it = fDSPUI->begin4(); it != fDSPUI->end4(); it++) {
         t_atom myList[4];
         atom_setsym(&myList[0], gensym((*it).first.c_str()));
-        atom_setfloat(&myList[1], (*it).second->getInitValue());
+        atom_setfloat(&myList[1], (*it).second->getInitValue());    // init value
         atom_setfloat(&myList[2], (*it).second->getMinValue());
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(m_control_outlet, 0, 4, myList);
     }
 }
 
-// Dump controllers as list of: [label, cur, min, max]
-void faustgen::dump(long inlet, t_symbol* s, long ac, t_atom* av)
+void faustgen::dump_inputs()
 {
     // Input controllers
-    for (mspUI::iterator it = fDSPUI->begin1(); it != fDSPUI->end1(); it++) {
+    for (mspUI::iterator it = fDSPUI->begin2(); it != fDSPUI->end2(); it++) {
         t_atom myList[4];
         atom_setsym(&myList[0], gensym((*it).first.c_str()));
-        atom_setfloat(&myList[1], (*it).second->getValue());
+        atom_setfloat(&myList[1], (*it).second->getValue());    // cur value
         atom_setfloat(&myList[2], (*it).second->getMinValue());
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(m_control_outlet, 0, 4, myList);
     }
+}
+void faustgen::dump_outputs()
+{
     // Output controllers
-    for (mspUI::iterator it = fDSPUI->begin3(); it != fDSPUI->end3(); it++) {
+    for (mspUI::iterator it = fDSPUI->begin4(); it != fDSPUI->end4(); it++) {
         t_atom myList[4];
         atom_setsym(&myList[0], gensym((*it).first.c_str()));
-        atom_setfloat(&myList[1], (*it).second->getValue());
+        atom_setfloat(&myList[1], (*it).second->getValue());    // cur value
         atom_setfloat(&myList[2], (*it).second->getMinValue());
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(m_control_outlet, 0, 4, myList);
     }
 }
 
+// Dump controllers as list of [path, cur, min, max]
+void faustgen::dump(long inlet, t_symbol* s, long ac, t_atom* av)
+{
+    dump_inputs();
+    dump_outputs();
+}
+
 // osc 'IP inport outport xmit bundle'
 void faustgen::osc(long inlet, t_symbol* s, long ac, t_atom* av)
 {
     if (ac == 5) {
-        if (fDSPfactory->lock()) {
-            
+        fDSPfactory->lock_audio();
+        fDSPfactory->lock_ui();
+        {
             delete fOSCUI;
             
             const char* argv1[32];
@@ -1365,10 +1376,9 @@ void faustgen::osc(long inlet, t_symbol* s, long ac, t_atom* av)
             fOSCUI->run();
             
             post(fOSCUI->getInfos().c_str());
-            fDSPfactory->unlock();
-        } else {
-            post("Mutex lock cannot be taken...");
         }
+        fDSPfactory->unlock_ui();
+        fDSPfactory->unlock_audio();
     } else {
         post("Should be : osc 'IP inport outport xmit(0|1|2) bundle(0|1)'");
     }
@@ -1381,11 +1391,11 @@ void faustgen::midievent(long inlet, t_symbol* s, long ac, t_atom* av)
         int channel = (int)av[0].a_w.w_long & 0x0f;
         
         if (ac == 1) {
-            fDSPfactory->fMidiHandler.handleSync(0.0, av[0].a_w.w_long);
+            fMidiHandler.handleSync(0.0, av[0].a_w.w_long);
         } else if (ac == 2) {
-            fDSPfactory->fMidiHandler.handleData1(0.0, type, channel, av[1].a_w.w_long);
+            fMidiHandler.handleData1(0.0, type, channel, av[1].a_w.w_long);
         } else if (ac == 3) {
-            fDSPfactory->fMidiHandler.handleData2(0.0, type, channel, av[1].a_w.w_long, av[2].a_w.w_long);
+            fMidiHandler.handleData2(0.0, type, channel, av[1].a_w.w_long, av[2].a_w.w_long);
         }
     }
 }
@@ -1492,16 +1502,18 @@ void faustgen::update_sourcecode()
 // Process the signal data with the Faust module
 inline void faustgen::perform(int vs, t_sample** inputs, long numins, t_sample** outputs, long numouts)
 {
-    if (!fMute && fDSPfactory->try_lock()) {
+    if (!fMute && fDSPfactory->try_lock_audio()) {
         // Has to be tested again when the lock has been taken...
         if (fDSP) {
             fDSP->compute(vs, static_cast<FAUSTFLOAT**>(inputs), static_cast<FAUSTFLOAT**>(outputs));
             if (fOSCUI) fOSCUI->endBundle();
-            update_outputs();
+            //update_outputs();
+            // Use the right outlet to output messages
+            dump_outputs();
         }
-        // Done for fMIDIUI and fOSCUI
+        // Done for fMidiUI and fOSCUI
         GUI::updateAllGuis();
-        fDSPfactory->unlock();
+        fDSPfactory->unlock_audio();
     } else {
         // Write null buffers to outs
         for (int i = 0; i < numouts; i++) {
@@ -1582,7 +1594,7 @@ void faustgen::add_midihandler()
     // Polyphonic DSP is controlled by MIDI
     if (fDSPfactory->fPolyphonic) {
         mydsp_poly* poly = static_cast<mydsp_poly*>(fDSP);
-        fDSPfactory->fMidiHandler.addMidiIn(poly);
+        fMidiHandler.addMidiIn(poly);
     }
 }
 
@@ -1591,7 +1603,7 @@ void faustgen::remove_midihandler()
     // Polyphonic DSP is controlled by MIDI
     if (fDSPfactory->fPolyphonic) {
         mydsp_poly* poly = static_cast<mydsp_poly*>(fDSP);
-        fDSPfactory->fMidiHandler.removeMidiIn(poly);
+        fMidiHandler.removeMidiIn(poly);
     }
 }
 
@@ -1605,11 +1617,16 @@ void faustgen::init_controllers()
     
     // MIDI handling
     if (!fMidiUI) {
-        fMidiUI = new MidiUI(&fDSPfactory->fMidiHandler);
+        fMidiUI = new MidiUI(&fMidiHandler);
         add_midihandler();
         fDSP->buildUserInterface(fMidiUI);
     }
     
+    // State handling
+    if (!fSavedUI) {
+        fSavedUI = new SaveLabelUI();
+    }
+    
     // Soundfile handling
     if (fDSPfactory->fSoundUI) {
         if (fDSPfactory->fPolyphonic) {
@@ -1626,7 +1643,9 @@ void faustgen::init_controllers()
 
 void faustgen::create_dsp(bool init)
 {
-    if (fDSPfactory->lock()) {
+    fDSPfactory->lock_audio();
+    fDSPfactory->lock_ui();
+    {
         fDSP = fDSPfactory->create_dsp_aux();
         assert(fDSP);
         
@@ -1649,8 +1668,11 @@ void faustgen::create_dsp(bool init)
         
         setupIO(&faustgen::perform, &faustgen::init, fDSP->getNumInputs(), fDSP->getNumOutputs(), init);
         
+        // Setup m_midi_outlet MIDI output handler
+        fMidiHandler.m_midi_outlet = m_midi_outlet;
+        
         // Load old controller state
-        fDSP->buildUserInterface(&fSavedUI);
+        fDSP->buildUserInterface(fSavedUI);
         
         // Possibly restart IO
         if (dspstate) {
@@ -1659,11 +1681,9 @@ void faustgen::create_dsp(bool init)
         
         // Send JSON to JS script
         create_jsui();
-        
-        fDSPfactory->unlock();
-    } else {
-        post("Mutex lock cannot be taken...");
     }
+    fDSPfactory->unlock_ui();
+    fDSPfactory->unlock_audio();
 }
 
 void faustgen::set_dirty()
@@ -1753,16 +1773,6 @@ void faustgen::mute(long inlet, long mute)
     fMute = mute;
 }
 
-/*
- // For Max 6
- #ifdef WIN32
- //extern "C" int main(void)
- extern "C" void ext_main(void* r)
- #else
- int main(void)
- #endif
- */
-
 extern "C" void ext_main(void* r)
 {
 #ifdef WIN32
@@ -1775,7 +1785,7 @@ extern "C" void ext_main(void* r)
     t_class * mclass = faustgen::makeMaxClass("faustgen~");
     post("faustgen~ v%s (sample = 64 bits code = %s)", FAUSTGEN_VERSION, getCodeSize());
     post("LLVM powered Faust embedded compiler v%s", getCLibFaustVersion());
-    post("Copyright (c) 2012-2019 Grame");
+    post("Copyright (c) 2012-2021 Grame");
     
     // Start 'libfaust' in multi-thread safe mode
     startMTDSPFactories();
